;***********************************************************
;(c) 2003 David Prutchi and Michael Norris
;
;file name = docd.asm
; Revision 2.0
;
;Purpose: Implement a controller for the impedance measurement 
;         circuit using the biphasic pulses, for the PIC16F84.
;
;Clock Frequency = 4MHZ
;
;************************************************************

	LIST	p=16F84,R=DEC,F=INHX8M
	
	#include "P16F84.INC"

	__CONFIG _XT_OSC&_WDT_OFF&_PWRTE_ON&_CP_OFF
	__IDLOCS 0200			;Version 2
	ERRORLEVEL 1,-302

#define	BANK0	BCF	STATUS,RP0
#define	BANK1	BSF	STATUS,RP0

;***********************************************************
; DATA MEMORY DEFINITIONS START HERE
;***********************************************************
RAM_START	EQU	0x0C

W_TEMP		EQU	RAM_START
STATUS_TEMP	EQU	W_TEMP+1
FSR_TEMP	EQU	STATUS_TEMP+1
DELAY		EQU	FSR_TEMP+1
;***********************************************************
; END OF MEMORY DEFINITIONS
;***********************************************************

;***********************************************************
; I/O PORT DEFINITIONS
;***********************************************************
;************************************************************
;PORTB (BITS 1-7) IS USED TO CONTROL THE STATE OF THE ANALOG 
;SWITCHES. PORTB,0 IS THE EXTERNAL INTERRUPT INPUT 
;(EXTERNAL TRIGGER)
;************************************************************
#define		IMPED_GND	PORTB,1
#define		IMPED_FLOAT	PORTB,2
#define		IMPED_SH	PORTB,3
#define		PHASE_1		PORTB,4
#define		IMPED_DSCHG	PORTB,5
#define		PHASE_2		PORTB,6
#define		IMPED_CHG	PORTB,7
;***********************************************************
;***********************************************************
;PORTA,0 IS THE INTERNAL_TRIG_NOT INPUT
;PULLING THIS LINE LOW WILL USE THE INTERNAL TIMER TO 
;TRIGGER THE MEASUREMENT SEQUENCE. EACH TIME THAT TIMER 0 ROLLS 
;OVER AN INTERRUPT IS GENERATED, AND A MEASUREMENT SEQUENCE IS 
;EXECUTED. THIS IS THE DEFAULT MODE.
;IF THIS LINE IS LEFT HIGH, ONLY AN EXTERNAL PULSE ON RB0 WILL 
;CAUSE A MEASUREMENT SEQUENCE TO BE EXECUTED.
;***********************************************************
#define		INTERNAL_TRIG_NOT	PORTA,0
;***********************************************************

;***********************************************************
; RESET VECTOR
;***********************************************************
	ORG	0x00
	GOTO	STARTUP
;***********************************************************
;INTERRUPT VECTOR
;***********************************************************
	ORG	0x04
	MOVWF	W_TEMP			;SAVE REGISTERS
	SWAPF	STATUS,W
	MOVWF	STATUS_TEMP

	BANK0
	MOVF	FSR,W
	MOVWF	FSR_TEMP

;INTERRUPT SERVICE ROUTINE STARTS HERE
	BTFSS	INTCON,INTF		;WAS IT AN EXTERNAL INTERRUPT?
	GOTO	TEST_TIMER_INT		;NO THEN TEST THE TIMER INTERRUPT

	BCF	INTCON,INTF		;YES THEN CLEAR THE INTERRUPT FLAG
	CALL	DO_MEAS			;AND DO A MEASUREMENT
	GOTO	EXIT_INT		;THEN EXIT.

TEST_TIMER_INT
	BTFSS	INTCON,T0IF		;WAS THIS A TMR0 INTERRUPT?
	GOTO	EXIT_INT		;NO THEN WHAT WAS IT? GO TO EXIT
					;YES THEN ...

	BCF	INTCON,T0IF		;CLEAR THE TMR0 INTERRUPT FLAG
	BTFSS	INTERNAL_TRIG_NOT	;IS THE INTERNAL TRIGGER LINE LOW?
	CALL	DO_MEAS			;YES THEN DO A SELF-TRIGGERED MEASUREMENT
					;NO THEN EXIT

EXIT_INT
	BANK0				;RESTORE REGISTERS
	MOVF	FSR_TEMP,W
	MOVWF	FSR
	SWAPF	STATUS_TEMP,W
	MOVWF	STATUS
	SWAPF	W_TEMP,F
	SWAPF	W_TEMP,W
	
	RETFIE				;AND RETURN

;***********************************************************
; INITIALIZATION ROUTINE
;***********************************************************
STARTUP
	BANK0

	MOVLW	0x00		;SETUP THE INITIAL STATE OF PORTB
	MOVWF	PORTB		;DATA REGISTER

	BSF	IMPED_CHG	;STARTUP WITH CHARGE ON, AND GND ON
	BSF	IMPED_GND
	
	BANK1			;MAKE PORTB ALL OUT EXCEPT RB0 WHICH IS AN INPUT
	MOVLW	0x01
	MOVWF	TRISB

	MOVLW	0xFF		;MAKE PORTA ALL INPUTS
	MOVWF	TRISA		

	MOVLW	b'11000011'	;PRESCALE 1:16, PULL-UPS OFF,INTERRUPT ON RISING EDGE
	MOVWF	OPTION_REG	;PRESCALER ASSIGNED TO TMR0

	BANK0
	CLRF	TMR0		;RESET TMR0 (TIMEOUT = 1us * 1:16 PRESCALE * 256 = 4.096ms)
	CLRF	INTCON		;CLEAR ALL INTERRUPT FLAGS AND INTERRUPTS
	
	BSF	INTCON,T0IE	;ENABLE TMR0 INTERRUPTS	
	BSF	INTCON,INTE	;ENABLE EXTERNAL INTERRUPTS
	BSF	INTCON,GIE	;ENABLE GLOBAL INTERRUPTS

;*****************************************************************
; MAIN STARTS HERE
;*****************************************************************
MAIN
				;DO NOTHING JUST
	GOTO	MAIN		;WAIT FOR INTERRUPTS

;*****************************************************************
; END OF MAIN
;*****************************************************************



;***************************************************************************
; DO_MEAS SUBROUTINE
;***************************************************************************
; This routine is called either from an external interrupt (external trigger) 
; or a (internal trigger) timer timeout if PORTA,0 is low. The timer timeout 
; is every 4.096ms. 
; This routine is ONLY called from the interrupt service routine.
;
; A summary of the measurement sequence is as follows:
; 1. Switch the cap. charge circuit off
; 2. Output a 10us negative pulse 
; 3. Output a 10us positive pulse 
; 4. Sample the voltage left on the two pulse caps.
; 5. Start the active discharge (discharge the series cap.)
; 6. Wait 768us for the sample and hold amplifier to stabilize
; 7. Stop sampling the cap. voltage and hold
; 8. Stop the active discharge
; 9. Start the pulse caps. charging
; 10. return
;***************************************************************************
; CALLS: NONE
; CALLED BY: THE INTERRUPT SERVICE ROUTINE
; RETURNS: NONE
; INPUTS: NONE
; VARIABLES USED: DELAY
;***************************************************************************
DO_MEAS
	BANK0
	BCF	IMPED_CHG	;CHARGE OFF
	BCF	IMPED_GND	;GROUND OFF

;***********************************************************************
;PHASE 2 PULSE
;***********************************************************************
	BSF	IMPED_FLOAT	;FLOAT ON (NOT FLOATING, CONNECTED)
	BSF	PHASE_2		;PHASE 2 PULSE ON

	NOP	;DO NOTHING FOR 9us
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	BCF	PHASE_2		;TURN OFF PHASE 2 PULSE (1us + 9us = 10us)
;***********************************************************************
; PHASE 1 PULSE
;***********************************************************************
	BSF	PHASE_1		;PHASE 1 PULSE ON

	NOP	;DO NOTHING FOR 9us
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	BCF	PHASE_1		;TURN OFF PHASE 1 PULSE
	BCF	IMPED_FLOAT	;FLOAT OFF (FLOATING, NOT CONNECTED)

;*************************************************************************
; 1. START THE ACTIVE DISCHARGE
; 2. GND ON FOR MEASUREMENT
; 3. START THE SAMPLE AND HOLD (SAMPLE)
; 4. WAIT FOR THE INA102 TO STABILIZE BEFORE SWITCHING TO HOLD
;**************************************************************************	

	BSF	IMPED_DSCHG	;DISCHARGE ON

	BSF	IMPED_GND	;GROUND ON
	BSF	IMPED_SH	;SAMPLE AND HOLD ON (SAMPLE)

	CLRF	DELAY		;DELAY FOR 256 * 1us * 3 cycles
WAIT_SH
	DECFSZ	DELAY,F		;1us
	GOTO	WAIT_SH		;2us

;****************************************************************
; HOLD THE OUTPUT VOLTAGE
;****************************************************************
	BCF	IMPED_SH	;SAMPLE AND HOLD OFF (HOLD)
	BCF	IMPED_DSCHG	;DISCHARGE OFF
	BSF	IMPED_CHG	;CHARGE ON, CHARGE UP FOR THE NEXT CYCLE, 
				;GROUND IS ALREADY ON, NO NEED TO TURN IT ON.	
	RETURN			;EXIT BACK TO INTERRUPT ROUTINE.

;*****************************************************************
; END OF DO_MEAS ROUTINE
;*****************************************************************

	END
