% HODGKIN-HUXLEY RESPONSE OF A SQUID AXON TO STIMULUS
%
% Adapted from a MATLAB program by Prof. Mark E. Nelson
% for the course Physiology 317 - Methods in Computational Neurobiology
% at the University of Illinois, Urbana-Champaign
%
% Maximum Membrane Conductances
% -----------------------------
g_Na_max = 120.0;       % Maximum Sodium conductance in mSiemens/cm^2
g_K_max = 36.0;	        % Maximum Potassium conductance in mSiemens/cm^2
g_leak = 0.3;	        % Leakage conductance in mSiemens/cm^2
%
% Equilibrium Potentials for Ionic Species
% ----------------------------------------
E_Na = 115.0;	    % Sodium equilibrium potential in mV
E_K = -12.0;	    % Potassium equilibrium potential in mV
E_leak = 10.613;	% Leakage equilibrium potential in mV
%
% Membrane Capacitance
% --------------------
C_membrane = 1.0;	% Membrane capacitance uF/cm^2
%
% Simulation Parameters
% ---------------------
stimulus_list = [1.0 10.0 -1.0 -10.0];  % Stimulus currents in uA
dt = 0.025;         % Integration step in ms
t_max = 50;         % Time limit in ms
%
% Initialize Variables
% --------------------
t = 0:dt:t_max;     % Generate time vector
V_rest = 0.0;       % Initialize resting voltage
%
% SIMULATION
% ----------
for ival = 1:4
   I = stimulus_list(ival)*(t>10 & t<=40);  % Inject stimulus between 10ms and 40ms
   clear V_membrane  m  h  n			    % Clear varibles
   V_membrane = V_rest;		    			% Initial membrane voltage is resting potential
   %
   % Calculate steady-state rate constants
   % -------------------------------------
   m = alpha_m(V_membrane)/(alpha_m(V_membrane)+beta_m(V_membrane));	% Calculate steady-state m
   h = alpha_h(V_membrane)/(alpha_h(V_membrane)+beta_h(V_membrane));	% Calculate steady-state h
   n = alpha_n(V_membrane)/(alpha_n(V_membrane)+beta_n(V_membrane));	% Calculate steady-state n
   for i=2:length(t)
      %
      % Calculate Sodium and Potassium Conductances
      % -------------------------------------------
      g_Na = g_Na_max * (m(i-1))^3 * h(i-1);
      g_K  = g_K_max  * (n(i-1))^4;
      %
      % Calculate Rates of Change
      % -------------------------
      mdot = alpha_m(V_membrane(i-1))*(1-m(i-1)) - beta_m(V_membrane(i-1))*m(i-1);
      hdot = alpha_h(V_membrane(i-1))*(1-h(i-1)) - beta_h(V_membrane(i-1))*h(i-1);
      ndot = alpha_n(V_membrane(i-1))*(1-n(i-1)) - beta_n(V_membrane(i-1))*n(i-1);
      V_membrane_dot = (I(i-1) - g_Na*(V_membrane(i-1)-E_Na) - g_K*(V_membrane(i-1)-E_K) - g_leak*(V_membrane(i-1)-E_leak))/C_membrane;
      %
      % Integrate
      % ---------
      m(i) = m(i-1) + mdot*dt;
      h(i) = h(i-1) + hdot*dt;
      n(i) = n(i-1) + ndot*dt;
      V_membrane(i) = V_membrane(i-1) + V_membrane_dot*dt;
   end
   subplot(2,2,ival);
   plot(t,V_membrane,'b-'); hold on;
   plot(t,I,'r:')
   str = sprintf('Stimulus = %2.2f \\muA/cm^2',stimulus_list(ival));
   title(str);
   drawnow;
end
%
% Label Plots
% -----------
subplot(221); 
ylabel('V_m_e_m_b_r_a_n_e [mV]')
xlabel('time [ms]')
axis([0 t_max -2 2])
subplot(223); 
ylabel('V_m_e_m_b_r_a_n_e [mV]')
xlabel('time [ms]')
axis([0 t_max -2 2])
subplot(222); 
ylabel('V_m_e_m_b_r_a_n_e [mV]')
xlabel('time [ms]')
axis([0 t_max -50 150])
subplot(224); 
ylabel('V_m_e_m_b_r_a_n_e [mV]')
xlabel('time [ms]')
axis([0 t_max -50 150])
