/* Parameters 
	Mode				: VVI, VOO
	Frequency			: 40, 50, 55, 60, 65, 70, 80, 90, 100, 110, 120, 140 ppm 
	Refractory Period	: 250 and 350 ms
	Amplitude			: Vdd and 2*Vdd
	Pulse Width			: from 0.125 to 1.5 ms in steps of 0.125 ms 
	Sensitivity			: from 1.0 to 6.0 mv in steps of 1.0 mV
*/

#include <pic.h>
#include "ssi_ext.h"

// Periods (in ms) corresponding to the following frequencies (in ppm): 40, 50, 55, 60, 65, 70, 80, 90, 100, 110, 120, 140
unsigned int Tbl_Period[] = { 
	T_1500ms, T_1200ms, T_1091ms, T_1000ms, T_923ms, T_857ms, 
	T_750ms , T_667ms , T_600ms , T_545ms , T_500ms, T_429ms 
} ;
#define LAST_PERIOD		(sizeof(Tbl_Period)/sizeof(unsigned int))

// Pulses Width (in ms): from 0.125 to 1.5 ms in steps of 0.125 ms 
unsigned int Tbl_PWidth[] = {
	T_0_125ms, T_0_250ms, T_0_375ms, T_0_500ms, T_0_625ms, T_0_750ms,
	T_0_875ms, T_1_000ms, T_1_125ms, T_1_250ms, T_1_375ms, T_1_500ms
} ;
#define LAST_PWIDTH		(sizeof(Tbl_PWidth)/sizeof(unsigned int))

// Global Variables
BitsPORTA	bPORTA	@ (&PORTA) ;
BitsPORTA	bCopyPORTA ;
BitsPORTB	bPORTB	@ (&PORTB) ;
BitsPORTB	bCopyPORTB ;
BitsPORTC	bPORTC	@ (&PORTC) ;
BitsPORTC	bCopyPORTC ;

unsigned int Timer @ (&TMR1L) ;
unsigned int Tout, LastTout ;
unsigned int SenseTime ;
STATE State ;
EVENT Event ;

// Parameters
bit ModeVVI ;
bit HighAmplitude ;
unsigned int RefractoryTout ;
unsigned int PeriodTout ;
unsigned int PWidth ;

//////////////////////////////
/// Functions and Routines ///
//////////////////////////////

// Sensing routines
void EnableSense() 
{
	if( !ModeVVI )
		return ;
	if( !bPORTB.SensingIn )
		INTF = 0 ;
	INTE = 1 ;		// Enable INT interrupt
}

void DisableSense()
{
	INTE = 0 ;		// Disable INT interrupt
	INTF = 0 ;		// Clear INTF flag
}

// Timer routines
void InitTout( unsigned int t ) 
{
	LastTout = t ;
	TMR1ON = 0 ;
	Timer = -t ;
	TMR1ON = 1 ;
	TMR1IF = 0 ;
}

unsigned int Elapsed()
{
	unsigned int Result ;

	TMR1ON = 0 ;
	Result = Timer + LastTout ;
	TMR1ON = 1 ;
	return Result ;
}

int ReadTimer()
{
	int t ;
	
	TMR1ON = 0 ;
	t = Timer ;
	TMR1ON = 1 ;
	return t ;
}

void Pace()
{
	InitTout( PWidth ) ;
	bCopyPORTB.HighAmplitudePacing = HighAmplitude ;
	bPORTB = bCopyPORTB ;
	bCopyPORTC.Pacing = 1 ;
	bPORTC = bCopyPORTC ;
	Sleep() ;
	bCopyPORTC.Pacing = 0 ;
	bPORTC = bCopyPORTC ;
	bCopyPORTB.HighAmplitudePacing = 0  ;
	bPORTB = bCopyPORTB ;
	// Start QuickDischarge 
	bCopyPORTC.QuickDischarge = 0 ;
	bPORTC = bCopyPORTC ;
}

unsigned char GetAD( unsigned char ADChannel )
{
	unsigned char i ;
	
	ADCON0 = ADChannel ;
	ADON = 1 ;		// Turn on AD
	for( i = 0; i < 15; ++i) 
		asm("NOP") ;		// Waiting because of the hardware ...
	ADGO = 1 ;				// Start AD
	while ( ADGO ) ;
	ADON = 0 ;				// Turn off AD
	return ADRES ;
}

void ReadParameters()
{
	unsigned char AD ;

	bCopyPORTB.ReadingEnable = 0 ;				// 0 - Enable parameter reading
	bPORTB = bCopyPORTB ;

	if( bPORTB.Refractory_SelIn )				// 0 - 250 ms, 1 - 350 ms
		RefractoryTout = T_350ms  ;
	else
		RefractoryTout = T_250ms ;
	HighAmplitude = bPORTB.Amplitude_SelIn ;	// 0 - VDD, 1 - 2*VDD
	ModeVVI = bPORTB.Mode_SelIn ;				// 0 - VOO, 1 - VVI

	AD = GetAD( AD_FREQ_SEL_IN ) >> 4 ;			// from 0:255 to 0:15
	if( AD >= LAST_PERIOD )
		AD = LAST_PERIOD - 1 ;
	PeriodTout = Tbl_Period[ AD ] ;

	AD = GetAD( AD_PWIDTH_SEL_IN ) >> 4 ;		// from 0:255 to 0:15
	if( AD >= LAST_PWIDTH )
		AD = LAST_PERIOD - 1 ;
	PWidth = Tbl_PWidth[ AD ] ;

	bCopyPORTB.ReadingEnable = 1 ;				// 1 - Disable parameter reading
	bPORTB = bCopyPORTB ;
}

EVENT GetEvent()
{
	InitTout( Tout );
	Sleep();
	if( INTF && INTE )				//	Sensing event
	{
		INTF = 0;
		SenseTime = ReadTimer();
		return SENSE;
	}
	else if ( TMR1IF && TMR1IE )	// timeout
	{
		TMR1IF = 0;
		return TOUT;
	}
	else
		return UNKNOWN_EVENT;
}

main() 
{
	unsigned int TimeStamp ;
	
	__CONFIG(0x3F3B) ;	// CodeProtect = OFF, Oscillator = RC, WatchDog = OFF

	STATUS = 0x00 ;
	INTCON = 0x00 ;		// All interrupts disabled
	OPTION = 0xC0 ;		// Pull Up disable, Clock internal, WDT off

	// Configure Timer1
	T1CON  = 0x0E ;		// Timer1 as asynch clock
	Timer = 0 ;
	TMR1ON = 1 ;		// Turn On
	TMR1IE = 1 ;		// Enable Timer 1 wake up

		
	// Set the AD converter
	ADCON1 = 0x04 ;		// RA0, RA1 and RA3  A/D pins

	PEIE   = 1 ;		// Enable Peripheral Interrupts

	// Set the hardware I/O pins
	*((char *)&bCopyPORTA) = DefaultPortA ;
	*((char *)&bCopyPORTB) = DefaultPortB ;
	*((char *)&bCopyPORTC) = DefaultPortC ;

	bPORTA = bCopyPORTA ;
	bPORTB = bCopyPORTB ;	
	bPORTC = bCopyPORTC ;

					//	Pin  7   6   5   4   3   2   1   0
	TRISA = 0x03 ;	//		XXX XXX Out Out Out Out Inp Inp
	TRISB = 0x71 ;	//      Out Inp Inp Inp Out Out Out Inp
	TRISC = 0x01 ;	//      Out Out Out Out Out Out Out Inp

	// Read the parameters
	ReadParameters() ;
	Tout = RefractoryTout - T_100ms ;
	DisableSense() ;
	State =  REFRACTORY ;

	for(;;)
	{
		switch( State ) 
		{
		case ALERT:
			Event = GetEvent();
			DisableSense() ;	
			switch( Event )
			{
			case TOUT:
				Pace();
				Tout = RefractoryTout - T_100ms ;
				State = REFRACTORY;
				break;
			case SENSE:
				if ( (TimeStamp == 0) || (Elapsed() - TimeStamp > T_100ms) )	// TimeStamp is a "negative number"
				{
					Tout = RefractoryTout - T_100ms;
					State = REFRACTORY;
				}
				else 
				{
					Tout = Tout - Elapsed() ;
					State = WAIT_END_ALERT ;
				}
				break;
			default:
				State = FAILURE;
				break;
			}
			break;
		case WAIT_END_ALERT:
			Event = GetEvent();
			switch( Event ) 
			{
			case TOUT:
				Pace();
				Tout = RefractoryTout - T_100ms ;
				State = REFRACTORY;
				break;
			default:
				State = FAILURE;
				break;
			}
			break;
		case REFRACTORY:
			ReadParameters();
			Event = GetEvent();
			// End QuickDischarge 
			bCopyPORTC.QuickDischarge = 1 ;
			bPORTC = bCopyPORTC ;
			switch( Event )
			{
			case TOUT:
				EnableSense();
				TimeStamp = 0 ;
				Tout = T_100ms ;
				State = NOISE_WINDOW;
				break;
			default:
				State = FAILURE;
				break;
			}
			break;
		case NOISE_WINDOW:
			Event = GetEvent();
			switch( Event ) 
			{
			case TOUT:
				Tout = PeriodTout - RefractoryTout ;
				State = ALERT ;
				break;
			case SENSE:
				TimeStamp = SenseTime ;
				Tout = Tout - Elapsed() ;
				break;
			default:
				State = FAILURE;
				break;
			}
			break;
		case FAILURE:
		default:
			break;
		}
	}
}
